/*
 * Base.c
 *
 * Copyright (C) 2007 Amos Brocco
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * You should have received a copy of the GNU General Public License
 * (for example COPYING); if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <glib/gprintf.h>
#include "Base.h"
#include "GSuperMacro.h"

// /////////////////////////////////////////////////////////////////////////////////////////////////
// PRIVATE STRUCTURE
// /////////////////////////////////////////////////////////////////////////////////////////////////

struct _BasePrivate {

};

// /////////////////////////////////////////////////////////////////////////////////////////////////
// PRIVATE FUNCTION PROTOTYPES
// /////////////////////////////////////////////////////////////////////////////////////////////////

static void base_finalize (GObject* object);

static void base_dispose (GObject* object);

static gint real_sample_method(Base* self, gint value);

static void real_another_method(Base* self);

// /////////////////////////////////////////////////////////////////////////////////////////////////
// TYPE DEFINITION
// /////////////////////////////////////////////////////////////////////////////////////////////////

G_DEFINE_TYPE (Base, base, G_TYPE_OBJECT)

// /////////////////////////////////////////////////////////////////////////////////////////////////
// CONSTRUCTORS
// /////////////////////////////////////////////////////////////////////////////////////////////////

static void base_class_init(BaseClass* klass) {
	GObjectClass *g_object_class;

	/* Add private structure */
	g_type_class_add_private(klass, sizeof(BasePrivate));

	/* Get the parent gobject class */
	g_object_class = G_OBJECT_CLASS(klass);

	/* Hook public methods */
	G_HOOK_METHOD(klass,sample_method,real_sample_method);
	G_HOOK_METHOD(klass,another_method,real_another_method);

	/* Hook finalization functions */
	g_object_class->dispose = base_dispose;
	g_object_class->finalize = base_finalize;
}

static void base_init(Base* self) {
}

// /////////////////////////////////////////////////////////////////////////////////////////////////
// DESTRUCTORS
// /////////////////////////////////////////////////////////////////////////////////////////////////

static void base_finalize(GObject* object) {
	G_OBJECT_CLASS (base_parent_class)->finalize (object);
}

static void base_dispose(GObject* object) {
	G_OBJECT_CLASS (base_parent_class)->dispose (object);
}

// /////////////////////////////////////////////////////////////////////////////////////////////////
// PUBLIC METHODS
// /////////////////////////////////////////////////////////////////////////////////////////////////

gint base_sample_method(Base* self, int value) {
	return BASE_GET_CLASS (self)->sample_method (self, value);
}

void base_another_method(Base* self) {
	BASE_GET_CLASS (self)->another_method (self);
}

// /////////////////////////////////////////////////////////////////////////////////////////////////
// PRIVATE / STATIC METHODS
// /////////////////////////////////////////////////////////////////////////////////////////////////

static gint real_sample_method(Base* self, gint value) {
        g_printf("Sample method got %d\n", value);
	return value;
}

static void real_another_method(Base* self) {
        g_printf("Another method called\n");
}

