/*
 * Base.h
 *
 * Copyright (C) 2007 Amos Brocco
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * You should have received a copy of the GNU General Public License
 * (for example COPYING); if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#ifndef BASE_H_
#define BASE_H_

#include <glib-object.h>
#include <glib.h>

// /////////////////////////////////////////////////////////////////////////////////////////////////
// PRIVATE STRUCTURE PROTOTYPE
// /////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _BasePrivate BasePrivate;

// /////////////////////////////////////////////////////////////////////////////////////////////////
// INSTANCE STRUCTURE
// /////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _Base {
        GObject parent_instance;

        /* Pointer to private data structure */
        BasePrivate* priv;
} Base;

// /////////////////////////////////////////////////////////////////////////////////////////////////
// CLASS STRUCTURE
// /////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct _BaseClass {
        GObjectClass parent_class;

        /* Overridable public methods */
        gint (*sample_method) (Base* self, gint value);
        void (*another_method) (Base* self);

} BaseClass;

// /////////////////////////////////////////////////////////////////////////////////////////////////
// GTYPE BOILERPLATE
// /////////////////////////////////////////////////////////////////////////////////////////////////

#define TYPE_BASE               (base_get_type())
#define BASE(object)            (G_TYPE_CHECK_INSTANCE_CAST((object), TYPE_BASE, Base))
#define BASE_CLASS(klass)       (G_TYPE_CHECK_CLASS_CAST((klass), TYPE_BASE, BaseClass))
#define IS_BASE(object)         (G_TYPE_CHECK_INSTANCE_TYPE((object), TYPE_BASE))
#define IS_BASE_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE((klass), TYPE_BASE))
#define BASE_GET_CLASS(object)  (G_TYPE_INSTANCE_GET_CLASS((object), TYPE_BASE, BaseClass))
#define BASE_GET_PRIVATE(o)     (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_BASE, BasePrivate))

// /////////////////////////////////////////////////////////////////////////////////////////////////
// PUBLIC METHODS PROTOTYPES
// /////////////////////////////////////////////////////////////////////////////////////////////////

gint     base_sample_method      (Base* self, gint value);
void     base_another_method     (Base* self);

#endif /*BASE_H_*/

