/*
 * Sub.c
 *
 * Copyright (C) 2007 Amos Brocco
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * You should have received a copy of the GNU General Public License
 * (for example COPYING); if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "Sub.h"
#include "Base.h"
#include "GSuperMacro.h"
#include <glib/gprintf.h>

// /////////////////////////////////////////////////////////////////////////////////////////////////
// PRIVATE STRUCTURE
// /////////////////////////////////////////////////////////////////////////////////////////////////

struct _SubPrivate {
};

// /////////////////////////////////////////////////////////////////////////////////////////////////
// PRIVATE FUNCTION PROTOTYPES
// /////////////////////////////////////////////////////////////////////////////////////////////////

static void real_sub_another_method(Base* self);

static gint real_sub_sample_method(Base* self, gint value);

// /////////////////////////////////////////////////////////////////////////////////////////////////
// STATIC VARIABLES
// /////////////////////////////////////////////////////////////////////////////////////////////////

/* Define static variables for pointers to parent's methods */
G_SUPER_METHOD(gint,sample_method,(Base* self, gint value));
G_SUPER_METHOD(void,another_method,(Base* self));

// /////////////////////////////////////////////////////////////////////////////////////////////////
// TYPE DEFINITION
// /////////////////////////////////////////////////////////////////////////////////////////////////

G_DEFINE_TYPE (Sub, sub, TYPE_BASE)

// /////////////////////////////////////////////////////////////////////////////////////////////////
// CONSTRUCTORS
// /////////////////////////////////////////////////////////////////////////////////////////////////

static void sub_class_init(SubClass* klass) {
	/* Add private structure */
	g_type_class_add_private(klass, sizeof(SubPrivate));
	
	/* Override parent class methods (requires a G_SUPER_METHOD definition for each overridden method) */
	G_OVERRIDE_METHOD(BASE_CLASS(klass),sample_method,real_sub_sample_method);
	G_OVERRIDE_METHOD(BASE_CLASS(klass),another_method,real_sub_another_method);
}

static void sub_init(Sub* self) {
}

// /////////////////////////////////////////////////////////////////////////////////////////////////
// PRIVATE / STATIC METHODS
// /////////////////////////////////////////////////////////////////////////////////////////////////

static gint real_sub_sample_method(Base* self, gint value) {
        g_printf("Sub Sample method got %d\n", value);
        /* Call the super implementation of the method with the given parameters at last*/
	return G_SUPER(sample_method,(self, value));
}

static void real_sub_another_method(Base* self) {
        /* Call the super implementation of the method with the given parameters first */
        G_SUPER(another_method,(self));
        g_printf("Sub Another method called\n");
}

